'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var lezerJavascript = require('lezer-javascript');
var language = require('@codemirror/language');
var highlight = require('@codemirror/highlight');
var autocomplete = require('@codemirror/autocomplete');

/**
A collection of JavaScript-related
[snippets](https://codemirror.net/6/docs/ref/#autocomplete.snippet).
*/
const snippets = [
    autocomplete.snippetCompletion("function ${name}(${params}) {\n\t${}\n}", {
        label: "function",
        detail: "definition",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("for (let ${index} = 0; ${index} < ${bound}; ${index}++) {\n\t${}\n}", {
        label: "for",
        detail: "loop",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("for (let ${name} of ${collection}) {\n\t${}\n}", {
        label: "for",
        detail: "of loop",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("if (condition) {\n\t${}\n}", {
        label: "if",
        detail: "if condition",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("if (condition) {\n\t${}\n} else{\n\t\n}", {
        label: "ifelse",
        detail: "ifelse",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("switch (key) {\n\tcase value:\n\t\t${}\n\t\tbreak;\n\n\tdefault:\n\t\tbreak;\n}", {
        label: "switch",
        detail: "switch case",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("try {\n\t${}\n} catch (${error}) {\n\t${}\n}", {
        label: "try",
        detail: "block",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("class ${name} {\n\tconstructor(${params}) {\n\t\t${}\n\t}\n}", {
        label: "class",
        detail: "definition",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("import {${names}} from \"${module}\"\n${}", {
        label: "import",
        detail: "named",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("import ${name} from \"${module}\"\n${}", {
        label: "import",
        detail: "default",
        type: "keyword"
    }),
    autocomplete.snippetCompletion('console.log("${}")', {
        label: "console",
        detail: "default",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("await ", {
        label: "await",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("async ", {
        label: "async",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("abstract ", {
        label: "abstract",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("arguments ", {
        label: "arguments",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("boolean ", {
        label: "boolean",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("break ", {
        label: "break",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("byte ", {
        label: "byte",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("case ", {
        label: "case",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("catch ", {
        label: "catch",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("char ", {
        label: "char",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("class ", {
        label: "class",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("const ", {
        label: "const",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("continue ", {
        label: "continue",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("debugger ", {
        label: "debugger",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("default ", {
        label: "default",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("delete ", {
        label: "delete",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("do ", {
        label: "do",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("double ", {
        label: "double",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("else ", {
        label: "else",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("enum ", {
        label: "enum",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("eval ", {
        label: "eval",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("export ", {
        label: "export",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("extends ", {
        label: "extends",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("false ", {
        label: "false",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("final ", {
        label: "final",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("finally ", {
        label: "finally",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("float ", {
        label: "float",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("for ", {
        label: "for",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("function ", {
        label: "function",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("goto ", {
        label: "goto",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("if ", {
        label: "if",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("implements ", {
        label: "implements",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("import ", {
        label: "import",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("in ", {
        label: "in",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("instanceof ", {
        label: "instanceof",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("int ", {
        label: "int",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("interface ", {
        label: "interface",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("let ", {
        label: "let",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("long ", {
        label: "long",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("native ", {
        label: "native",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("new ", {
        label: "new",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("null ", {
        label: "null",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("package ", {
        label: "package",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("private ", {
        label: "private",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("protected ", {
        label: "protected",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("public ", {
        label: "public",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("return ", {
        label: "return",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("short ", {
        label: "short",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("static ", {
        label: "static",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("super ", {
        label: "super",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("switch ", {
        label: "switch",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("synchronized ", {
        label: "synchronized",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("this ", {
        label: "this",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("throw ", {
        label: "throw",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("throws ", {
        label: "throws",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("transient ", {
        label: "transient",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("true ", {
        label: "true",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("try ", {
        label: "try",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("typeof ", {
        label: "typeof",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("var ", {
        label: "var",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("void ", {
        label: "void",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("volatile ", {
        label: "volatile",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("while ", {
        label: "while",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("with ", {
        label: "with",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("yield ", {
        label: "yield",
        detail: "",
        type: "keyword"
    }),
    autocomplete.snippetCompletion("execJavaScript(`${}`) ", {
        label: "execJavaScript",
        detail: "",
        type: "method"
    }),
    autocomplete.snippetCompletion("openBrowser(`${}`) ", {
        label: "openBrowser",
        info: "openBrowser(url:string)",
        detail: "",
        type: "method"
    }),
    autocomplete.snippetCompletion("sleep(${}) ", {
        label: "sleep(ms)",
        info: function sleep() {
            let hint = document.createElement('span');
            hint.style.fontSize = '12px';
            hint.style.color = 'rgb(148 148 148)';
            hint.innerText = '# 休眠(ms)';
            return hint;
        },
        detail: ":void",
        type: "method"
    }),
    autocomplete.snippetCompletion("hint(`${}`) ", {
        label: "hint",
        info: "提示信息",
        detail: "",
        type: "method"
    }),
    autocomplete.snippetCompletion("dom${}", {
        label: "dom",
        info: "class",
        detail: "",
        type: "class"
    }),
    autocomplete.snippetCompletion("dom.get(`${}`)", {
        label: "dom.get",
        info: "get(sign:string)",
        detail: "",
        type: "method"
    }),
    autocomplete.snippetCompletion("dom.getByText(`${}`)", {
        label: "dom.getByText",
        info: "getByText(text:string,index:int)",
        detail: "",
        type: "method"
    }),
    autocomplete.snippetCompletion("dom.wait(`${}`)", {
        label: "dom.wait",
        info: "wait(sign:string,ms:int)",
        detail: "",
        type: "method"
    }),
    autocomplete.snippetCompletion("dom.click(`${}`)", {
        label: "dom.click",
        info: "click()",
        detail: "",
        type: "method"
    }),
    autocomplete.snippetCompletion("dom.set(`${}`)", {
        label: "dom.set",
        info: "set(value:string)",
        detail: "",
        type: "method"
    }),
    autocomplete.snippetCompletion("dom.reSet(`${}`)", {
        label: "dom.reSet",
        info: "reSet(value:string)",
        detail: "",
        type: "method"
    }),
    autocomplete.snippetCompletion("dom.clear(`${}`)", {
        label: "dom.clear",
        info: "clear()",
        detail: "",
        type: "method"
    }),
    autocomplete.snippetCompletion("dom.upload(`${}`)", {
        label: "dom.upload",
        info: "upload(id:string,filePath:string)",
        detail: "",
        type: "method"
    }),
    autocomplete.snippetCompletion("dom.selectText(${})", {
        label: "dom.selectText",
        info: "selectText(start:int,end:int)",
        detail: "",
        type: "method"
    }),
    autocomplete.snippetCompletion("dom.getAttributes(`${}`)", {
        label: "dom.getAttributes",
        info: "getAttributes(name:string)",
        detail: "",
        type: "method"
    }),
    autocomplete.snippetCompletion("dom.lineKeyOperation(`${}`)", {
        label: "dom.lineKeyOperation",
        info: "lineKeyOperation(key:string,rowKey:string,optKey:string)",
        detail: "",
        type: "method"
    }),
    autocomplete.snippetCompletion("page${}", {
        label: "page",
        info: "class",
        detail: "",
        type: "class"
    }),
    autocomplete.snippetCompletion("page.to(`${}`)", {
        label: "page.to",
        info: "to(url:string)",
        detail: "",
        type: "method"
    }),
    autocomplete.snippetCompletion("page.create(`${}`)", {
        label: "page.create",
        info: "create(url:string)",
        detail: "",
        type: "method"
    }),
    autocomplete.snippetCompletion("page.delete(${})", {
        label: "page.delete",
        info: "delete(index:int)",
        detail: "",
        type: "method"
    }),
    autocomplete.snippetCompletion("page.change(${})", {
        label: "page.change",
        info: "change(index:int)",
        detail: "",
        type: "method"
    }),
    autocomplete.snippetCompletion("page.changeIframe(${})", {
        label: "page.changeIframe",
        info: "changeIframe(index:int)",
        detail: "",
        type: "method"
    }),
    autocomplete.snippetCompletion("keyboard${}", {
        label: "keyboard",
        info: "class",
        detail: "",
        type: "class"
    }),
    autocomplete.snippetCompletion("keyboard.press(`${}`)", {
        label: "keyboard.press",
        info: "press(keyCode:string)",
        detail: "",
        type: "method"
    }),
    autocomplete.snippetCompletion("keyboard.down(`${}`)", {
        label: "keyboard.down",
        info: "down(keyCode:string)",
        detail: "",
        type: "method"
    }),
    autocomplete.snippetCompletion("keyboard.up(`${}`)", {
        label: "keyboard.up",
        info: "up(keyCode:string)",
        detail: "",
        type: "method"
    }),
    autocomplete.snippetCompletion("keyboard.type(`${}`)", {
        label: "keyboard.type",
        info: "type(keyCode:string)",
        detail: "",
        type: "method"
    }),
    autocomplete.snippetCompletion("mouse${}", {
        label: "mouse",
        info: "class",
        detail: "",
        type: "class"
    }),
    autocomplete.snippetCompletion("mouse.down(`${}`)", {
        label: "mouse.down",
        info: "down(options:{button?:string;clickCount?:number;})",
        detail: "",
        type: "method"
    }),
    autocomplete.snippetCompletion("mouse.up(`${}`)", {
        label: "mouse.up",
        info: "up(options:{button?:string;clickCount?:number;})",
        detail: "",
        type: "method"
    }),
    autocomplete.snippetCompletion("mouse.click(`${}`)", {
        label: "mouse.click",
        info: "click(x:number,y:number,options:{delay?:number;})",
        detail: "",
        type: "method"
    }),
    autocomplete.snippetCompletion("mouse.move(`${}`)", {
        label: "mouse.move",
        info: "move(options:{button?:string;clickCount?:number;})",
        detail: "",
        type: "method"
    }),
    autocomplete.snippetCompletion("mouse.wheel(`${}`)", {
        label: "mouse.wheel",
        info: "wheel(options:{button?:string;clickCount?:number;})",
        detail: "",
        type: "method"
    }),
    autocomplete.snippetCompletion("assert${}", {
        label: "assert",
        info: "class",
        detail: "",
        type: "class"
    }),
    autocomplete.snippetCompletion("assert.all(`${}`)", {
        label: "assert.all",
        info: "all(text:string)",
        detail: "",
        type: "method"
    }),
    autocomplete.snippetCompletion("assert.custom(`${}`)", {
        label: "assert.custom",
        info: "custom(type:string,dom:string,operate:int,result:int)",
        detail: "",
        type: "method"
    }),
];

/**
A language provider based on the [Lezer JavaScript
parser](https://github.com/lezer-parser/javascript), extended with
highlighting and indentation information.
*/
const javascriptLanguage = language.LezerLanguage.define({
    parser: lezerJavascript.parser.configure({
        props: [
            language.indentNodeProp.add({
                IfStatement: language.continuedIndent({ except: /^\s*({|else\b)/ }),
                TryStatement: language.continuedIndent({ except: /^\s*({|catch|finally)\b/ }),
                LabeledStatement: language.flatIndent,
                SwitchBody: context => {
                    let after = context.textAfter, closed = /^\s*\}/.test(after), isCase = /^\s*(case|default)\b/.test(after);
                    return context.baseIndent + (closed ? 0 : isCase ? 1 : 2) * context.unit;
                },
                Block: language.delimitedIndent({ closing: "}" }),
                ArrowFunction: cx => cx.baseIndent + cx.unit,
                "TemplateString BlockComment": () => -1,
                "Statement Property": language.continuedIndent({ except: /^{/ }),
                JSXElement(context) {
                    let closed = /^\s*<\//.test(context.textAfter);
                    return context.lineIndent(context.state.doc.lineAt(context.node.from)) + (closed ? 0 : context.unit);
                },
                JSXEscape(context) {
                    let closed = /\s*\}/.test(context.textAfter);
                    return context.lineIndent(context.state.doc.lineAt(context.node.from)) + (closed ? 0 : context.unit);
                },
                "JSXOpenTag JSXSelfClosingTag"(context) {
                    return context.column(context.node.from) + context.unit;
                }
            }),
            language.foldNodeProp.add({
                "Block ClassBody SwitchBody EnumBody ObjectExpression ArrayExpression": language.foldInside,
                BlockComment(tree) { return { from: tree.from + 2, to: tree.to - 2 }; }
            }),
            highlight.styleTags({
                "get set async static": highlight.tags.modifier,
                "for while do if else switch try catch finally return throw break continue default case": highlight.tags.controlKeyword,
                "in of await yield void typeof delete instanceof": highlight.tags.operatorKeyword,
                "export import let var const function class extends": highlight.tags.definitionKeyword,
                "with debugger from as new": highlight.tags.keyword,
                TemplateString: highlight.tags.special(highlight.tags.string),
                Super: highlight.tags.atom,
                BooleanLiteral: highlight.tags.bool,
                this: highlight.tags.self,
                null: highlight.tags.null,
                Star: highlight.tags.modifier,
                VariableName: highlight.tags.variableName,
                "CallExpression/VariableName": highlight.tags.function(highlight.tags.variableName),
                VariableDefinition: highlight.tags.definition(highlight.tags.variableName),
                Label: highlight.tags.labelName,
                PropertyName: highlight.tags.propertyName,
                "CallExpression/MemberExpression/PropertyName": highlight.tags.function(highlight.tags.propertyName),
                "FunctionDeclaration/VariableDefinition": highlight.tags.function(highlight.tags.definition(highlight.tags.variableName)),
                "ClassDeclaration/VariableDefinition": highlight.tags.definition(highlight.tags.className),
                PropertyNameDefinition: highlight.tags.definition(highlight.tags.propertyName),
                UpdateOp: highlight.tags.updateOperator,
                LineComment: highlight.tags.lineComment,
                BlockComment: highlight.tags.blockComment,
                Number: highlight.tags.number,
                String: highlight.tags.string,
                ArithOp: highlight.tags.arithmeticOperator,
                LogicOp: highlight.tags.logicOperator,
                BitOp: highlight.tags.bitwiseOperator,
                CompareOp: highlight.tags.compareOperator,
                RegExp: highlight.tags.regexp,
                Equals: highlight.tags.definitionOperator,
                "Arrow : Spread": highlight.tags.punctuation,
                "( )": highlight.tags.paren,
                "[ ]": highlight.tags.squareBracket,
                "{ }": highlight.tags.brace,
                ".": highlight.tags.derefOperator,
                ", ;": highlight.tags.separator,
                TypeName: highlight.tags.typeName,
                TypeDefinition: highlight.tags.definition(highlight.tags.typeName),
                "type enum interface implements namespace module declare": highlight.tags.definitionKeyword,
                "abstract global privacy readonly": highlight.tags.modifier,
                "is keyof unique infer": highlight.tags.operatorKeyword,
                JSXAttributeValue: highlight.tags.string,
                JSXText: highlight.tags.content,
                "JSXStartTag JSXStartCloseTag JSXSelfCloseEndTag JSXEndTag": highlight.tags.angleBracket,
                "JSXIdentifier JSXNameSpacedName": highlight.tags.tagName,
                "JSXAttribute/JSXIdentifier JSXAttribute/JSXNameSpacedName": highlight.tags.propertyName
            })
        ]
    }),
    languageData: {
        closeBrackets: { brackets: ["(", "[", "{", "'", '"', "`"] },
        commentTokens: { line: "//", block: { open: "/*", close: "*/" } },
        indentOnInput: /^\s*(?:case |default:|\{|\}|<\/)$/,
        wordChars: "$"
    }
});
/**
A language provider for TypeScript.
*/
const typescriptLanguage = javascriptLanguage.configure({ dialect: "ts" });
/**
Language provider for JSX.
*/
const jsxLanguage = javascriptLanguage.configure({ dialect: "jsx" });
/**
Language provider for JSX + TypeScript.
*/
const tsxLanguage = javascriptLanguage.configure({ dialect: "jsx ts" });
/**
JavaScript support. Includes [snippet](https://codemirror.net/6/docs/ref/#lang-javascript.snippets)
completion.
*/
function cherry(config = {}) {
    let lang = config.jsx ? (config.typescript ? tsxLanguage : jsxLanguage)
        : config.typescript ? typescriptLanguage : javascriptLanguage;
    return new language.LanguageSupport(lang, javascriptLanguage.data.of({
        autocomplete: autocomplete.ifNotIn(["LineComment", "BlockComment", "String"], autocomplete.completeFromList(snippets))
    }));
}

/**
Connects an [ESLint](https://eslint.org/) linter to CodeMirror's
[lint](https://codemirror.net/6/docs/ref/#lint) integration. `eslint` should be an instance of the
[`Linter`](https://eslint.org/docs/developer-guide/nodejs-api#linter)
class, and `config` an optional ESLint configuration. The return
value of this function can be passed to [`linter`](https://codemirror.net/6/docs/ref/#lint.linter)
to create a JavaScript linting extension.

Note that ESLint targets node, and is tricky to run in the
browser. The [eslint4b](https://github.com/mysticatea/eslint4b)
and
[eslint4b-prebuilt](https://github.com/marijnh/eslint4b-prebuilt/)
packages may help with that.
*/
function esLint(eslint, config) {
    if (!config) {
        config = {
            parserOptions: { ecmaVersion: 2019, sourceType: "module" },
            env: { browser: true, node: true, es6: true, es2015: true, es2017: true, es2020: true },
            rules: {}
        };
        eslint.getRules().forEach((desc, name) => {
            if (desc.meta.docs.recommended)
                config.rules[name] = 2;
        });
    }
    return (view) => {
        let { state } = view, found = [];
        for (let { from, to } of javascriptLanguage.findRegions(state)) {
            let fromLine = state.doc.lineAt(from), offset = { line: fromLine.number - 1, col: from - fromLine.from, pos: from };
            for (let d of eslint.verify(state.sliceDoc(from, to), config))
                found.push(translateDiagnostic(d, state.doc, offset));
        }
        return found;
    };
}
function mapPos(line, col, doc, offset) {
    return doc.line(line + offset.line).from + col + (line == 1 ? offset.col - 1 : -1);
}
function translateDiagnostic(input, doc, offset) {
    let start = mapPos(input.line, input.column, doc, offset);
    let result = {
        from: start,
        to: input.endLine != null && input.endColumn != 1 ? mapPos(input.endLine, input.endColumn, doc, offset) : start,
        message: input.message,
        source: input.ruleId ? "jshint:" + input.ruleId : "jshint",
        severity: input.severity == 1 ? "warning" : "error",
    };
    if (input.fix) {
        let { range, text } = input.fix, from = range[0] + offset.pos - start, to = range[1] + offset.pos - start;
        result.actions = [{
                name: "fix",
                apply(view, start) {
                    view.dispatch({ changes: { from: start + from, to: start + to, insert: text }, scrollIntoView: true });
                }
            }];
    }
    return result;
}

exports.cherry = cherry;
exports.esLint = esLint;
exports.javascriptLanguage = javascriptLanguage;
exports.jsxLanguage = jsxLanguage;
exports.snippets = snippets;
exports.tsxLanguage = tsxLanguage;
exports.typescriptLanguage = typescriptLanguage;
